"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BuildKit = void 0;
const fs_1 = __importDefault(require("fs"));
const core = __importStar(require("@actions/core"));
const exec = __importStar(require("@actions/exec"));
const semver = __importStar(require("semver"));
const buildx_1 = require("./buildx");
const builder_1 = require("./builder");
class BuildKit {
    constructor(opts) {
        this.containerNamePrefix = 'buildx_buildkit_';
        this.context = opts.context;
        this.buildx =
            (opts === null || opts === void 0 ? void 0 : opts.buildx) ||
                new buildx_1.Buildx({
                    context: this.context
                });
    }
    getBuilderInfo(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const builder = new builder_1.Builder({
                context: this.context,
                buildx: this.buildx
            });
            return builder.inspect(name);
        });
    }
    getVersion(builderName) {
        return __awaiter(this, void 0, void 0, function* () {
            const builderInfo = yield this.getBuilderInfo(builderName);
            if (builderInfo.nodes.length == 0) {
                // a builder always have on node, should not happen.
                return undefined;
            }
            // TODO: get version for all nodes
            const node = builderInfo.nodes[0];
            if (!node.buildkitVersion && node.name) {
                try {
                    return yield this.getVersionWithinImage(node.name);
                }
                catch (e) {
                    core.warning(e);
                }
            }
            return node.buildkitVersion;
        });
    }
    getVersionWithinImage(nodeName) {
        return __awaiter(this, void 0, void 0, function* () {
            return exec
                .getExecOutput(`docker`, ['inspect', '--format', '{{.Config.Image}}', `${this.containerNamePrefix}${nodeName}`], {
                ignoreReturnCode: true,
                silent: true
            })
                .then(bkitimage => {
                if (bkitimage.exitCode == 0 && bkitimage.stdout.length > 0) {
                    return exec
                        .getExecOutput(`docker`, ['run', '--rm', bkitimage.stdout.trim(), '--version'], {
                        ignoreReturnCode: true,
                        silent: true
                    })
                        .then(bkitversion => {
                        if (bkitversion.exitCode == 0 && bkitversion.stdout.length > 0) {
                            return `${bkitimage.stdout.trim()} => ${bkitversion.stdout.trim()}`;
                        }
                        else if (bkitversion.stderr.length > 0) {
                            throw new Error(bkitimage.stderr.trim());
                        }
                        return bkitversion.stdout.trim();
                    });
                }
                else if (bkitimage.stderr.length > 0) {
                    throw new Error(bkitimage.stderr.trim());
                }
                return bkitimage.stdout.trim();
            });
        });
    }
    versionSatisfies(builderName, range) {
        return __awaiter(this, void 0, void 0, function* () {
            const builderInfo = yield this.getBuilderInfo(builderName);
            for (const node of builderInfo.nodes) {
                let bkversion = node.buildkitVersion;
                if (!bkversion) {
                    try {
                        bkversion = yield this.getVersionWithinImage(node.name || '');
                    }
                    catch (e) {
                        return false;
                    }
                }
                // BuildKit version reported by moby is in the format of `v0.11.0-moby`
                if (builderInfo.driver == 'docker' && !bkversion.endsWith('-moby')) {
                    return false;
                }
                if (!semver.satisfies(bkversion.replace(/-moby$/, ''), range)) {
                    return false;
                }
            }
            return true;
        });
    }
    generateConfigInline(s) {
        return this.generateConfig(s, false);
    }
    generateConfigFile(s) {
        return this.generateConfig(s, true);
    }
    generateConfig(s, file) {
        if (file) {
            if (!fs_1.default.existsSync(s)) {
                throw new Error(`config file ${s} not found`);
            }
            s = fs_1.default.readFileSync(s, { encoding: 'utf-8' });
        }
        const configFile = this.context.tmpName({ tmpdir: this.context.tmpDir() });
        fs_1.default.writeFileSync(configFile, s);
        return configFile;
    }
}
exports.BuildKit = BuildKit;
//# sourceMappingURL=buildkit.js.map